﻿// MZpack 3 Pro API demo
// Copyright © 2020 MZpack Co.
// www.mzpack.pro
// WARNING: to compile this sample in NinjaTrader 8 remove '#if APISAMPLE' and '#endif' directives
#if APISAMPLE
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using MZpack;
using MZpack.NT8;
using MZpack.NT8.Algo;
using MZpack.NT8.Algo.Extensions.Entries;
using MZpack.NT8.Algo.Extensions.Trails;
using MZpack.NT8.Algo.Indicators;
using MZpack.NT8.Algo.Signals;
using NinjaTrader.Data;
using NinjaTrader.Gui;

namespace NinjaTrader.NinjaScript.Strategies.MZpackAPISamples
{
    [CategoryOrder("Signals", 2)]
    [CategoryOrder("Filters", 3)]
    [CategoryOrder("Position", 4)]
    [CategoryOrder("Visual", 5)]
    /// <summary>
    /// Demonstrates algo strategy class MZpack.NT8.Algo.Strategy and built-in TradesClusterSignal, DOMImbalanceSignal classes.
    /// Implements Partially Visible mode of the indicators.
    /// </summary>
    public class MZpackAlgoStrategy2 : MZpackStrategyBase
    {
        // Indicators
        StrategyBigTradeIndicator bigTradeIndicator;
        StrategyFootprintIndicator footPrintIndicator;
        StrategyMarketDepthIndicator domImbalanceIndicator;

        // Signal name
        static readonly string SIGNAL_NAME = @"Algo #2";

        public MZpackAlgoStrategy2() : base()
        {
            // Create MZpack algo strategy object to use built-in pattern and position management
            OnCreateAlgoStrategy = new OnCreateAlgoStrategyDelegate(CreateAlgoStrategy);

            // Set OnCreateIndicators delegate
            OnCreateIndicators = new OnCreateIndicatorsDelegate(CreateIndicators);
        }

        protected MZpack.NT8.Algo.Strategy CreateAlgoStrategy()
        {
            return new MZpack.NT8.Algo.Strategy(@"MZpack Algo Strategy 2", this)
            {
                LogLevel = LogLevel, // Set log level of algo from strategy UI
                LogTarget = LogTarget, // Set log target of algo from strategy UI
                LogTime = LogTime
            };
        }

        protected List<TickIndicator> CreateIndicators()
        {
            // Initialize new indicators list
            List<TickIndicator> indicators = new List<TickIndicator>();

            // Create BigTrade indicator
            bigTradeIndicator = new StrategyBigTradeIndicator(this, @"Clusters")
            {
                // Display filters
                FilterLogic = TradeFilterLogic.ALL,
                TradeFilterEnable = true,
                DomPressureFilterEnable = true,
                // Presentation
                MarkerType = TradeMarkerType.Box,
                MarkerPosition = TradeMarkerPosition.POC,
                MaxShapeExtent = 10,
                ColorMode = ColorMode.Saturation,
                ShowTradePOC = true,
                VolumesPosition = VolumesPosition.OutsideRight,
                // Alerts
                UseBigTradeAlert = false
            };
            indicators.Add(bigTradeIndicator);

            footPrintIndicator = new StrategyFootprintIndicator(this, @"Absorptions")
            {
                LeftFootprintStyle = FootprintStyle.None,
                RightFootprintStyle = FootprintStyle.None,
                ShowSessionPOC = false,
                ShowSessionVA = false,
                ShowImbalance = false,
                ShowBarVA = false,
                ShowBarVolume = false,
                ShowBarCOT = false,
                ShowBarDelta = false,
                ShowBarDeltaPercent = false,
                ShowBarMinMaxDelta = false,
                ShowAbsorptionSRZones = true,
                AbsorptionPercentage = 50,
                AbsorptionSRZonesConsecutiveLevels = 2,
                AbsorptionSRZonesVolumeFilter = 80,
                AbsoprtionSRZoneEnding = SRZoneEnding.ByBarClose,
                AbsorptionSRZonesBreakOnSession = false,
                StatisticGridShow = false
            };
            indicators.Add(footPrintIndicator);

            domImbalanceIndicator = new StrategyMarketDepthIndicator(this, @"DOM Imbalance")
            {
                ShowRealtimeOrderBook = false,
                ShowImbalance = true,
                ImbalancePercentage = Filters_DOMImbalance_Ratio * 100d - 100d // Conver ratio to percentage, eg. 1.2 = 20%
            };
            indicators.Add(domImbalanceIndicator);

            return indicators;
        }

        protected override void OnStateChange()
        {
            base.OnStateChange();  // Base OnStateChange() call is required

            try
            {
                if (State == State.SetDefaults)
                {
                    // Default values for parameters
                    Common_TraceOrders = false;
                    //
                    ShowPatternsDashboard = true;
                    DashboardGridShowLegend = true;
                    // Position allowed directions, quantity and stop/target
                    Position_Direction = SignalDirection.Any;
                    Position_Quantity = 2;
                    Position_StopLoss = 18;
                    Position_ProfitTarget = 35;
                    // Break even
                    Position_IsBE = false;
                    Position_BEAfterTicks = 14;
                    Position_BEShiftTicks = 4;
                    // Trailing
                    Position_IsTrail = true;
                    Position_TrailAfter = 20;
                    Position_TrailDistance = 10;
                    Position_TrailStep = 2;
                    // Signals
                    BigTrade_TradeMin = 20;
                    BigTrade_TradeMax = 500;
                    BigTrade_DomPressureMin = 5;
                    BigTrade_ClusterBarRange = 3;
                    BigTrade_ClusterPriceRangeTicks = 6;
                    BigTrade_MinTotalVolume = 50;
                    BigTrade_MinTotalCount = 3;
                    BigTrade_EnableAggression = false;
                    BigTrade_MinAggression = 1;
                    // Filters
                    Filters_DOMImbalance = true;
                    Filters_DOMImbalance_Ratio = 1.2;
                    // Visual
                    VisibleBigTradeIndicator = false;
                    PartiallyVisibleBigTradeIndicator = true;
                    VisibleFootPrintIndicator = false;
                    PartiallyVisibleFootPrintIndicator = true;
                    VisibleDomImbalanceIndicator = true;
                    PartiallyVisibleDomImbalanceIndicator = false;
                }
                else
                if (State == State.Configure)
                {
                    TraceOrders = Common_TraceOrders;
                    EntriesPerDirection = 1;

                    // Assign indicator's attributes from UI properties
                    bigTradeIndicator.TradeFilterMin = BigTrade_TradeMin;
                    bigTradeIndicator.TradeFilterMax = BigTrade_TradeMax;
                    bigTradeIndicator.DomPressureFilterMin = BigTrade_DomPressureMin;
                    bigTradeIndicator.AggressionFilterEnable = BigTrade_EnableAggression;
                    bigTradeIndicator.AggressionFilterMin = BigTrade_MinAggression;
                    bigTradeIndicator.Visible = VisibleBigTradeIndicator;
                    bigTradeIndicator.IsPartiallyVisible = PartiallyVisibleBigTradeIndicator;
                    footPrintIndicator.Visible = VisibleFootPrintIndicator;
                    footPrintIndicator.IsPartiallyVisible = PartiallyVisibleFootPrintIndicator;
                    domImbalanceIndicator.Visible = VisibleDomImbalanceIndicator;
                    domImbalanceIndicator.IsPartiallyVisible = PartiallyVisibleDomImbalanceIndicator;

                    // Create entries
                    Entry[] entries = new Entry[EntriesPerDirection];
                    entries[0] = new Entry(Strategy)
                    {
                        EntryMethod = EntryMethod.Market,
                        Quantity = Position_Quantity,
                        SignalName = SIGNAL_NAME,
                        StopLossTicks = Position_StopLoss,
                        ProfitTargetTicks = Position_ProfitTarget,
                        IsBreakEven = Position_IsBE,
                        BreakEvenAfterTicks = Position_BEAfterTicks,
                        BreakEvenShiftTicks = Position_BEShiftTicks,
                        Trail = Position_IsTrail ? new Trail(Position_TrailAfter, Position_TrailDistance, Position_TrailStep) : null
                    };

                    // Pattern
                    Pattern pattern = new Pattern(Strategy,
                        Logic.And, // Logic for root node of signals tree
                        new Range() { Bars = BigTrade_ClusterBarRange + 2, Ticks = BigTrade_ClusterPriceRangeTicks + 4 },  // Range for signals (0 = no range limit)
                        Logic.And, // Logic for root node of filters tree
                        new Range() { Bars = 5 }, // Range for filters
                        true); // Short-cuircit for AND evaluation

                    pattern.AllowedDirection = Position_Direction;  // Set allowed direction(s)

                    // Create signals tree
                    LogicalNode orClustersNode = new LogicalNode(Logic.Or);
                    pattern.Signals.Root.AddChild(orClustersNode);

                    // SHORT signal - cluster of buy trades
                    orClustersNode.AddChild(
                        new TradesClusterSignal(Strategy, bigTradeIndicator, SignalDirection.Short)
                        {
                            Name = "Ask Clusters",
                            Side = TradeSide.Ask,
                            Range = new Range() { Bars = BigTrade_ClusterBarRange, Ticks = BigTrade_ClusterPriceRangeTicks, Logic = Logic.And },
                            MinVolume = BigTrade_MinTotalVolume,
                            MinCount = BigTrade_MinTotalCount,
                            VolumeCountLogic = Logic.And,
                            MinAggression = BigTrade_MinAggression,
                            PartiallyVisibleMode = SignalPartiallyVisibleMode.OnSignalValidated
                        });

                    // LONG signal - cluster of sell trades
                    orClustersNode.AddChild(
                        new TradesClusterSignal(Strategy, bigTradeIndicator, SignalDirection.Long)
                        {
                            Name = "Bid Clusters",
                            Side = TradeSide.Bid,
                            Range = new Range() { Bars = BigTrade_ClusterBarRange, Ticks = BigTrade_ClusterPriceRangeTicks, Logic = Logic.And },
                            MinVolume = BigTrade_MinTotalVolume,
                            MinCount = BigTrade_MinTotalCount,
                            VolumeCountLogic = Logic.And,
                            MinAggression = BigTrade_MinAggression,
                            PartiallyVisibleMode = SignalPartiallyVisibleMode.OnSignalValidated
                        });

                    // LONG/SHORT Absoprtion SR zones signal
                    if (Footprint_EnableAbsorption)
                        pattern.Signals.Root.AddChild(new AbsorptionSRZonesSignal(Strategy, footPrintIndicator) { Name = "Absorption" });

                    if (Filters_DOMImbalance)
                        pattern.Filters.Root.AddChild(
                            new DOMImbalanceSignal(Strategy, domImbalanceIndicator)
                            {
                                Name = "DOM Imbalance",
                                IsReset = false,
                                HasPrice = false,
                                Ratio = Filters_DOMImbalance_Ratio
                            });

                    Strategy.Initialize(pattern, entries, 3);  // Initialize strategy
                }
                else
                if (State == State.Terminated)
                {
                    if (bigTradeIndicator != null)
                    {
                        VisibleBigTradeIndicator = bigTradeIndicator.Visible;
                        PartiallyVisibleBigTradeIndicator = bigTradeIndicator.IsPartiallyVisible;
                    }
                    if (footPrintIndicator != null)
                    {
                        VisibleFootPrintIndicator = footPrintIndicator.Visible;
                        PartiallyVisibleFootPrintIndicator = footPrintIndicator.IsPartiallyVisible;
                    }
                    if (domImbalanceIndicator != null)
                    { 
                        VisibleDomImbalanceIndicator = domImbalanceIndicator.Visible;
                        PartiallyVisibleDomImbalanceIndicator = domImbalanceIndicator.IsPartiallyVisible;
                    }
                }
            }
            catch (Exception ex)
            {
                HandleThrowExceptionAndTerminate(ex); // Handle errors. HandleThrowException() just prints and raises exception
            }
        }

        class AbsorptionSRZonesSignal : Signal
        {
            StrategyFootprintIndicator indicator;
            IFootprintBar bar;

            public AbsorptionSRZonesSignal(MZpack.NT8.Algo.Strategy strategy, StrategyFootprintIndicator indicator) : base(strategy, MarketDataSource.Level1, SignalCalculate.OnBarClose, false)
            {
                this.indicator = indicator;
            }

            public override void OnCalculate(MarketDataEventArgs e, int barIdx, SignalDirection allowed)
            {
                IFootprintBar _bar;
                if (indicator.FootprintBars.TryGetValue(barIdx, out _bar))
                {
                    int buyAbsCnt = _bar.AbsorptionSRZones.Zones[(int)TradeSide.Ask].Count;
                    int sellAbsCnt = _bar.AbsorptionSRZones.Zones[(int)TradeSide.Bid].Count;

                    SignalDirection dir;
                    if (buyAbsCnt > 0 && sellAbsCnt == 0)
                        dir = SignalDirection.Short;
                    else
                    if (sellAbsCnt > 0 && buyAbsCnt == 0)
                        dir = SignalDirection.Long;
                    else
                        dir = SignalDirection.None;

                    dir = Signal.ResolveDirection(dir, allowed);  // Resolve calculated direction using allowed one.

                    if (Signal.IsDetermined(dir))  // This signal must be LONG or SHORT only.
                    {
                        Direction = dir;
                        Time = e.Time;
                        EntryPrice = e.Price;
                        ChartRange = new ChartRange() { MinBarIdx = barIdx, MaxBarIdx = barIdx, Low = e.Price, High = e.Price };
                        Description = string.Format("{0}: Buy absoprtion zone(s) {1}, Sell absoprtion zone(s) {2}", GetType().Name, buyAbsCnt, sellAbsCnt);

                        bar = _bar;
                        // Show plots of the signal in partially visible mode during pattern validation process
                        bar.PartiallyVisible = true;
                    }
                }
            }

            protected override void Reset(bool resetView)
            {
                base.Reset(resetView);  // base call required

                if (bar != null)
                {
                    // Make footprint bar not visible in partially visible mode on signal reset
                    if (resetView)  // Check resetView state and (optional) PartiallyVisibleMode
                        bar.PartiallyVisible = false;
                    bar = null;
                }
            }

            // Show plots of the signal on validation of the whole pattern in partially visible mode
            //public override void OnPatternValidated()
            //{
            //    if (bar != null)
            //        bar.PartiallyVisible = true; // Show signal footprint bar in partially visible mode
            //}
        }


        // --------------------------------- Common
        [Display(Name = "Trace orders", GroupName = "Common", Order = 21, Description = "")]
        public bool Common_TraceOrders
        { get; set; }

        // ----------------------------- Position
        [NinjaScriptProperty]
        [Display(Name = "Direction", GroupName = "Position", Order = 0, Description = "")]
        public SignalDirection Position_Direction
        { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "Quantity", GroupName = "Position", Order = 1, Description = "")]
        [Range(1, int.MaxValue)]
        public int Position_Quantity
        { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "Stop loss, ticks", GroupName = "Position", Order = 3, Description = "")]
        [Range(1, int.MaxValue)]
        public int Position_StopLoss
        { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "Profit target, ticks", GroupName = "Position", Order = 5, Description = "")]
        [Range(1, int.MaxValue)]
        public int Position_ProfitTarget
        { get; set; }

        [Display(Name = "BE", GroupName = "Position", Order = 7, Description = "")]
        public bool Position_IsBE
        { get; set; }

        [Display(Name = "BE after, ticks", GroupName = "Position", Order = 10, Description = "")]
        [Range(1, int.MaxValue)]
        public int Position_BEAfterTicks
        { get; set; }

        [Display(Name = "BE shift, ticks", GroupName = "Position", Order = 11, Description = "")]
        public int Position_BEShiftTicks
        { get; set; }

        [Display(Name = "Trail entry", GroupName = "Position", Order = 12, Description = "")]
        public bool Position_IsTrail
        { get; set; }

        [Display(Name = "Trail after, ticks", GroupName = "Position", Order = 20, Description = "")]
        [Range(1, int.MaxValue)]
        public int Position_TrailAfter
        { get; set; }

        [Display(Name = "Trail distance, ticks", GroupName = "Position", Order = 21, Description = "")]
        [Range(1, int.MaxValue)]
        public int Position_TrailDistance
        { get; set; }

        [Display(Name = "Trail step, ticks", GroupName = "Position", Order = 22, Description = "")]
        [Range(1, int.MaxValue)]
        public int Position_TrailStep
        { get; set; }

        //--------------------------------------- Signals
        [NinjaScriptProperty]
        [Display(Name = "Trade: min volume", GroupName = "Signals", Order = 2, Description = "Trade min volume filter")]
        [Range(1, long.MaxValue)]
        public long BigTrade_TradeMin
        { get; set; } = 20;

        [NinjaScriptProperty]
        [Display(Name = "Trade: max volume", GroupName = "Signals", Order = 3, Description = "Trade min volume filter")]
        [Range(1, long.MaxValue)]
        public long BigTrade_TradeMax
        { get; set; } = 200;

        [Display(Name = "Trade: DOM pressure min", GroupName = "Signals", Order = 5, Description = "Dom Pressure min volume filter")]
        [Range(0, long.MaxValue)]
        public long BigTrade_DomPressureMin
        { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "Trade: cluster bar range", GroupName = "Signals", Order = 6, Description = "")]
        [Range(-1, int.MaxValue)]
        public int BigTrade_ClusterBarRange
        { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "Trade: cluster price range, ticks", GroupName = "Signals", Order = 7, Description = "")]
        [Range(-1, int.MaxValue)]
        public int BigTrade_ClusterPriceRangeTicks
        { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "Trade: min total volume", GroupName = "Signals", Order = 9, Description = "")]
        [Range(1, long.MaxValue)]
        public long BigTrade_MinTotalVolume
        { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "Trade: min count", GroupName = "Signals", Order = 9, Description = "")]
        [Range(1, int.MaxValue)]
        public int BigTrade_MinTotalCount
        { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "Trade: enable aggression", GroupName = "Signals", Order = 10, Description = "")]
        public bool BigTrade_EnableAggression
        { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "Trade: min aggression", GroupName = "Signals", Order = 11, Description = "")]
        [Range(1, int.MaxValue)]
        public int BigTrade_MinAggression
        { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "Absorption", GroupName = "Signals", Order = 12, Description = "")]
        public bool Footprint_EnableAbsorption
        { get; set; }

        //--------------------------------------- Filters
        [NinjaScriptProperty]
        [Display(Name = "DOM Imbalance", GroupName = "Filters", Order = 1, Description = "")]
        public bool Filters_DOMImbalance { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "DOM Imbalance: ratio", GroupName = "Filters", Order = 2, Description = "")]
        public double Filters_DOMImbalance_Ratio { get; set; }

        //---------------------------------------- Visual
        // Properties to save visibility states of indicators in Xml
        [Display(Name = "Clusters: visible", GroupName = "Visual", Order = 1000)]
        public bool VisibleBigTradeIndicator { get; set; }
        [Display(Name = "Clusters: partially visible", GroupName = "Visual", Order = 1001)]
        public bool PartiallyVisibleBigTradeIndicator { get; set; }
        [Display(Name = "Absorptions: visible", GroupName = "Visual", Order = 1002, Description = "")]
        public bool VisibleFootPrintIndicator { get; set; }
        [Display(Name = "Absorptions: partially visible", GroupName = "Visual", Order = 1003, Description = "")]
        public bool PartiallyVisibleFootPrintIndicator { get; set; }
        [Display(Name = "DOM Imbalance: visible", GroupName = "Visual", Order = 1004, Description = "")]
        public bool VisibleDomImbalanceIndicator { get; set; }
        [Display(Name = "DOM Imbalance: partially visible", GroupName = "Visual", Order = 1005, Description = "")]
        public bool PartiallyVisibleDomImbalanceIndicator { get; set; }
    }
}
#endif
