﻿// MZpack 3 Pro API demo
// Copyright © 2020 MZpack Co.
// www.mzpack.pro
// WARNING: to compile this sample in NinjaTrader 8 remove '#if APISAMPLE' and '#endif' directives
#if APISAMPLE
using System;
using System.Collections.Generic;
using MZpack;
using MZpack.NT8;
using MZpack.NT8.Algo;
using MZpack.NT8.Algo.Indicators;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using NinjaTrader.Gui;
using NinjaTrader.Data;

namespace NinjaTrader.NinjaScript.Strategies.MZpackAPISamples
{
    /// <summary>
    /// Demonstrates how to use StrategyVolumeProfileIndicator class, including backtesting in Strategy Analyzer.
    /// The logic of the strategy is 'On bar close'.
    /// ATTENTION: Indicator settings in UI are used only for displaying markers on the chart. Strategy uses its own internal settings for filters and logic.
    /// </summary>
    /// <summary>
    /// Strategy logic description.
    /// </summary>
    /// <summary>
    /// <summary>
    /// On bar close strategy.
    /// Enter position: enter SHORT if bar closes over VWAP deviation and over average POC of given number of last bars; enter LONG if bar closes below VWAP deviation and below average POC of given number of last bars;
    /// </summary>
    /// <summary>
    /// See installation folder for the source code.
    /// </summary>
    public class MZpackCustomStrategy0 : MZpackStrategyBase
    {
        StrategyVolumeProfileIndicator sessionVPIndicator;
        double runningPOC;


        public MZpackCustomStrategy0() : base()
        {
            // Set OnCreateIndicators delegate
            OnCreateIndicators = new OnCreateIndicatorsDelegate(CreateIndicators);

            // Set OnBarCloseHandler for our on bar close strategy
            OnBarCloseHandler = new OnTickDelegate(StrategyOnBarCloseHandler);
        }

        /// <summary>
        /// Create indicators here and return them in the list as a result.
        /// </summary>
        /// <returns>Indicators list</returns>
        protected List<TickIndicator> CreateIndicators()
        {
            // Initialize new indicators list
            List<TickIndicator> indicators = new List<TickIndicator>();

            // Create CustomVolumeProfileIndicator instance with default settings
            // See CustomVolumeProfileIndicator class
            sessionVPIndicator = new StrategyVolumeProfileIndicator(this, @"Session Volume Profile")
            {
                // Override defaults if required
                ProfileCreation = ProfileCreation.Session,
                ShowProfileType = ProfileType.VP,  // Volume profile only
                Sigma1 = VP_Sigma1,
                VWAPMode = VWAPMode.DynamicStdDev1,
                ProfileWidthPercentage = 20,
                POCMode = LevelMode.Developing,
                VAHVALMode = LevelMode.Off,

                // No stacked profiles
                StackedProfileCreation1 = ProfileCreation.None,
                StackedProfileCreation2 = ProfileCreation.None,
                StackedProfileCreation3 = ProfileCreation.None
            };
            // Add indicator to the list
            indicators.Add(sessionVPIndicator);  

            return indicators;
        }

        protected override void OnStateChange()
        {
            // Base OnStateChange() call is required
            base.OnStateChange();

            lock (Sync) // Sync OnStateChange handler
            {
                if (State == State.SetDefaults)
                {
                    EntriesPerDirection = 1;
                    BarsRequiredToTrade = 1;

                    // Set defaults
                    Strategy_AveragePOCBarsBack = 30;
                    Strategy_TakeProfitTicks = 90;
                    Strategy_StopLossTicks = 12;
                    VP_ProfileMode = ProfileMode.Volume;
                    VP_TicksPerLevel = 1;
                    VP_Sigma1 = 1;
                    VP_VWAPStroke = new Stroke(System.Windows.Media.Brushes.Red, DashStyleHelper.Dash, 3, 50);
                    VP_Deviation1Stroke = new Stroke(System.Windows.Media.Brushes.Green);

                    // This is required for backtesting in Strategy Analyzer
                    EnableBacktesting = true;
                }
                else if (State == State.Configure)
                {
                    sessionVPIndicator.Calculate = Calculate.OnBarClose;  // Calculate.OnBarClose is recommended for mzVolumeProfile for performance reasons if the logic of the strategy is OnBarClose

                    // When State = State.Configure we can set indicator attributes from UI properties
                    sessionVPIndicator.ProfileMode = VP_ProfileMode;
                    sessionVPIndicator.TicksPerLevel = VP_TicksPerLevel;
                    sessionVPIndicator.VWAPStroke = VP_VWAPStroke;
                    sessionVPIndicator.Deviation1Stroke = VP_Deviation1Stroke;
                }
            }
        }

        /// <summary>
        /// The handler triggered on bar close. currentBarIdx is the index of just closed bar.
        /// e belongs to the next (just opened) bar.
        /// </summary>
        protected void StrategyOnBarCloseHandler(MarketDataEventArgs e, int currentBarIdx)
        {
            // Always get last profile using current bar index in OnBarCloseHandler
            IVolumeProfile profile = (sessionVPIndicator.Profiles as Model).GetItemByBarIdx(currentBarIdx) as IVolumeProfile;

            if (profile != null)
            {
                // Deviations
                double positiveDeviation1 = profile.VWAP + profile.Deviation * sessionVPIndicator.Sigma1;
                double negativeDeviation1 = profile.VWAP - profile.Deviation * sessionVPIndicator.Sigma1;

                // Calculate average POC price for last Strategy_AveragePOCBarsBack bars
                if (profile.POCs.Count == 1)
                    runningPOC = 0;  // Reset running POC for new profile

                runningPOC += profile.POCs[currentBarIdx];  // Add current POC price
                if (profile.POCs.Count > Strategy_AveragePOCBarsBack)
                    runningPOC -= profile.POCs[currentBarIdx - (Strategy_AveragePOCBarsBack - 1)];  // Subtract POC price that is out of back bars range

                double averagePOC = runningPOC / Math.Min(Strategy_AveragePOCBarsBack, profile.POCs.Count);

                // Short
                if (Close[1] > positiveDeviation1 && averagePOC < Close[1])
                {
                    string signalShort = string.Format("stdDev1:{0:N2} av.POC:{1:N2}", positiveDeviation1, averagePOC);
                    SetProfitTarget(signalShort, CalculationMode.Ticks, Strategy_TakeProfitTicks);
                    SetStopLoss(signalShort, CalculationMode.Ticks, Strategy_StopLossTicks, false);
                    EnterShort(signalShort);
                }
                else
                // Long
                if (Close[1] < negativeDeviation1 && averagePOC > Close[1])
                {
                    string signalLong = string.Format("stdDev1:{0:N2} av.POC:{1:N2}", negativeDeviation1, averagePOC);
                    SetProfitTarget(signalLong, CalculationMode.Ticks, Strategy_TakeProfitTicks);
                    SetStopLoss(signalLong, CalculationMode.Ticks, Strategy_StopLossTicks, false);
                    EnterLong(signalLong);
                }
            }
        }


        [Display(Name = "Average POC bars back", GroupName = "00. Strategy", Order = 0, Description = "")]
        [Range(1, int.MaxValue)]
        [NinjaScriptProperty]
        public int Strategy_AveragePOCBarsBack
        { get; set; }

        [Display(Name = "Take profit, ticks", GroupName = "00. Strategy", Order = 1, Description = "")]
        [Range(1, int.MaxValue)]
        [NinjaScriptProperty]
        public int Strategy_TakeProfitTicks
        { get; set; } 

        // Maximal stop loss
        [Display(Name = "Stop loss, ticks", GroupName = "00. Strategy", Order = 2, Description = "")]
        [Range(1, int.MaxValue)]
        [NinjaScriptProperty]
        public int Strategy_StopLossTicks
        { get; set; }

        [Display(Name = "Profile mode", GroupName = "01. Volume Profile", Order = 1, Description = "")]
        public ProfileMode VP_ProfileMode
        { get; set; }

        [Display(Name = "Ticks per level", GroupName = "01. Volume Profile", Order = 2, Description = "")]
        public int VP_TicksPerLevel
        { get; set; }

        [Display(Name = "Sigma 1", GroupName = "01. Volume Profile", Order = 3, Description = "")]
        public int VP_Sigma1
        { get; set; }

        [Display(Name = "VWAP", GroupName = "01. Volume Profile", Order = 4, Description = "")]
        public Stroke VP_VWAPStroke
        { get; set; }

        
        [Display(Name = "1st Deviation", GroupName = "01. Volume Profile", Order = 5, Description = "")]
        public Stroke VP_Deviation1Stroke
        { get; set; }
    }
}
#endif