﻿// MZpack 3 Pro API demo
// Copyright © 2020 MZpack Co.
// www.mzpack.pro
// WARNING: to compile this sample in NinjaTrader 8 remove '#if APISAMPLE' and '#endif' directives
#if APISAMPLE
using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using NinjaTrader.Data;
using NinjaTrader.Cbi;
using NinjaTrader.Core.FloatingPoint;
using MZpack;
using MZpack.NT8;
using MZpack.NT8.Algo;
using MZpack.NT8.Algo.Indicators;

namespace NinjaTrader.NinjaScript.Strategies.MZpackAPISamples
{
    /// <summary>
    /// Demonstrates how to use StrategyMarketDepthIndicator and StrategyFootprintIndicator classes. Use it on ES 5 Min chart during RTH session.
    /// </summary>
    /// <summary>
    /// WARNING: The strategy works on Market replay or live data only because it uses Level 2 data.
    /// </summary>
    /// <summary>
    /// ATTENTION: Indicator settings in UI are used only for displaying markers on the chart. Strategy uses its own internal settings for filters and logic.
    /// </summary>
    /// <summary>
    /// Strategy logic description.
    /// </summary>
    /// <summary>
    /// <summary>
    /// 1. Pace of tape rule: Check if a bar has clusters with high pace of tape in 'Cluster range' range from bar high or low. Sum trades numbers from those clusters and compare with 'Min trades number in clusters' parameter.
    /// </summary>
    /// <summary>
    /// 2. Imbalance rule: Calculate DOM Imbalance and check if it is greater or equal to 'Imbalance Ratio'.
    /// </summary>
    /// <summary>
    /// 3. Large block rule: Check if DOM contains one or more blocks of size 'Block volume' no closer than 'Order distance, ticks' parameter from the current price.
    /// </summary>
    /// 4. Session POC rule: Check if current price is above session POC for SHORT or below it for LONG.
    /// </summary>
    /// <summary>
    /// 5. If Imbalance, block(s), pace clusters are on the same DOM side and Session POC rule has the same direction, open position in opposite direction.
    /// </summary>
    /// <summary>
    /// See installation folder for the source code.
    /// </summary>
    public class MZpackCustomStrategy4 : MZpackStrategyBase
    {
        StrategyMarketDepthIndicator marketDepthIndicator;
        StrategyFootprintIndicator footprintIndicator;
        bool tradesClusterHi, tradesClusterLo;
        MarketPosition directionBySessionPOC = MarketPosition.Flat;
        string signalLong, signalShort;


        public MZpackCustomStrategy4() : base()
        {
            // Set OnCreateIndicators delegate
            OnCreateIndicators = new OnCreateIndicatorsDelegate(CreateIndicators);

            // Set OnEachTickHandler for our on each tick strategy
            OnEachTickHandler = new OnTickDelegate(StrategyOnEachTickHandler);

            // Set OnMarketDepthHandler for the strategy that uses DOM events
            OnMarketDepthHandler = new OnMarketDepthDelegate(StrategyOnMarketDepthHandler);
        }

        protected List<TickIndicator> CreateIndicators()
        {
            // Initialize new indicators list
            List<TickIndicator> indicators = new List<TickIndicator>();

            marketDepthIndicator = new StrategyMarketDepthIndicator(this, @"Depth Imbalance")
            {
                // Override defaults if required
                ShowRealtimeOrderBook = false,
                ShowCumulativeLevelII = false,
                ShowHistoricalDOM = true,
                ShowImbalance = false,
                HoldLevels = true,  // Hold historical values so we can use e.g. 18 ticks distance from current price for 10 real-time DOM levels on ES

                // Display filters
                MarketDepthFilteringMode = MarketDepthFilteringMode.Absolute,
                DisplayVolume = Strategy_BlockVolumeFilter,
                ExtremeVolume = Strategy_BlockVolumeFilter * 2,
                // Presentation

                // Alerts
                UseAboveExpectedAlert = false,
            };
            indicators.Add(marketDepthIndicator);

            footprintIndicator = new StrategyFootprintIndicator(this, @"Clusters")
            {
                // Override defaults if required
                RightFootprintStyle = FootprintStyle.TradesNumber,
                RightClusterStyle = ClusterStyle.Histogram,
                LeftFootprintStyle = FootprintStyle.Delta,
                LeftClusterStyle = ClusterStyle.None,
                ShowBarVolume = false,
                ShowBarDelta = false,
                ShowBarAbsoluteDeltaAverage = false,
                ShowBarMinMaxDelta = false,
                ShowBarDeltaPercent = false,
                ShowBarCOT = false,
                ShowUnfinishedAuction = false,
                StatisticGridShow = false,
                ShowBarPOC = false,
                ShowBarVA = false,
                ShowImbalance = false,
                ShowAbsorption = false,
                ShowSessionPOC = true,
                ShowSessionVA = true,
                SessionDailyProfileMode = SessionDailyProfileMode.Session,
            };
            indicators.Add(footprintIndicator);

            return indicators;
        }

        protected override void OnStateChange()
        {
            // Base OnStateChange() call is required
            base.OnStateChange();

            lock (Sync) // Sync OnStateChange handler
            {
                if (State == State.SetDefaults)
                {
                    EntriesPerDirection = 1;

                    // Set defaults
                    Strategy_TakeProfitTicks = 33;
                    Strategy_StopLossTicks = 22;
                    Strategy_Ratio = 1.4;
                    Strategy_BlockVolumeFilter = 600;
                    Strategy_BlockDistanceTicks = 8;
                    Strategy_TradesNumber = 500;
                    Strategy_ClusterRange = 3;
                }
                else
                if (State == State.Configure)
                {
                    // Set indicator attributes from UI properties
                    marketDepthIndicator.DisplayVolume = Strategy_BlockVolumeFilter;
                    marketDepthIndicator.ExtremeVolume = (long)(Strategy_BlockVolumeFilter * 1.5);
                }
            }
        }
                

        protected void StrategyOnEachTickHandler(MarketDataEventArgs e, int currentBarIdx)
        {
            IFootprintBar bar;
            if (footprintIndicator.FootprintBars.TryGetValue(currentBarIdx, out bar))
            {
                double priceLo = bar.Lo, priceHi = bar.Hi;  // Start from low and high of the bar
                int tradesOnLo = 0, tradesOnHi = 0;

                // Cummulate trades from high and low of the bar in Check Strategy_ClusterRange range
                for (int i = 0; i < Strategy_ClusterRange; i++)
                {
                    int n;
                    bar.TradesNumbers.TryGetValue(priceLo, out n);  // From low end of the bar
                    tradesOnLo += n;
                    bar.TradesNumbers.TryGetValue(priceHi, out n);  // From high end of the bar
                    tradesOnHi += n;

                    // Get next prices of clusters
                    priceLo = footprintIndicator.RoundToTickSize(priceLo + TickSize);
                    priceHi = footprintIndicator.RoundToTickSize(priceHi - TickSize);
                }

                // Set flags for clusters with high pace of tape
                tradesClusterLo = tradesOnLo >= Strategy_TradesNumber;
                tradesClusterHi = tradesOnHi >= Strategy_TradesNumber;

                // Compare current session POC with current price
                directionBySessionPOC = e.Price.ApproxCompare(bar.Session.POC) > 0 ? MarketPosition.Short : MarketPosition.Long;
            }
        }

        protected void StrategyOnMarketDepthHandler(MarketDepthEventArgs e, int currentBarIdx)
        {
            // Enter position
            if (Position.MarketPosition == Cbi.MarketPosition.Flat && // No position
                signalLong == null && signalShort == null // No orders has placed yet
                && (tradesClusterLo || tradesClusterHi))
            {
                long bid = marketDepthIndicator.RealtimeOrderBook.GetVolume(TradeSide.Bid);
                long offer = marketDepthIndicator.RealtimeOrderBook.GetVolume(TradeSide.Ask);

                if (bid > 0 && offer > 0)
                {
                    if (bid > offer)
                    {
                        double ratio = (double)bid / offer;
                        if (ratio > Strategy_Ratio && tradesClusterLo && directionBySessionPOC == MarketPosition.Long && OrderbookOrderRule(MarketPosition.Long))
                        {
                            signalLong = string.Format("LONG. Ratio:{0:N2}", ratio);
                            SetProfitTarget(signalLong, CalculationMode.Ticks, Strategy_TakeProfitTicks);
                            SetStopLoss(signalLong, CalculationMode.Ticks, Strategy_StopLossTicks, false);
                            EnterLong(signalLong);
                        }
                    }
                    else
                    if (bid < offer)
                    {
                        double ratio = (double)bid / offer;
                        if (ratio > Strategy_Ratio && tradesClusterHi && directionBySessionPOC == MarketPosition.Short && OrderbookOrderRule(MarketPosition.Short))
                        {
                            signalShort = string.Format("SHORT. Ratio:{0:N2}", ratio);
                            SetProfitTarget(signalShort, CalculationMode.Ticks, Strategy_TakeProfitTicks);
                            SetStopLoss(signalShort, CalculationMode.Ticks, Strategy_StopLossTicks, false);
                            EnterShort(signalShort);
                        }
                    }
                }
            }
        }

        bool OrderbookOrderRule(MarketPosition direction)
        {
            bool res = false;
            if (marketDepthIndicator.Blocks.ContainsKey(CurrentBar))
            {
                TradeSide depthSide = direction == MarketPosition.Long ? TradeSide.Bid : TradeSide.Ask;
                double price = direction == MarketPosition.Long ? GetCurrentBid() : GetCurrentAsk();

                List<KeyValuePair<double, IMarketDepthBlock>> blocks = marketDepthIndicator.Blocks[CurrentBar].Where(
                    x => GetDistanceTicks(price, x.Key, depthSide) <= Strategy_BlockDistanceTicks &&
                    x.Value.Volume >= Strategy_BlockVolumeFilter).ToList();

                res = blocks.Count > 0;
            }
            return res;
        }

        int GetDistanceTicks(double price, double depth, TradeSide side)
        {
            if (side == TradeSide.Bid && depth > price)
                return int.MaxValue;
            else
            if (side == TradeSide.Ask && depth < price)
                return int.MaxValue;

            return (int)(footprintIndicator.RoundToTickSize(Math.Abs(price - depth)) / TickSize);
        }

        // 
        protected override void OnOrderUpdate(Order order, double limitPrice, double stopPrice, int quantity, int filled, double averageFillPrice, OrderState orderState, DateTime time, ErrorCode error, string comment)
        {
            // Order cancelled/rejected
            if (orderState == OrderState.Cancelled || orderState == OrderState.CancelSubmitted || orderState == OrderState.Rejected)
            {
                if (order.Name == signalLong)
                    signalLong = null;
                else
                if (order.Name == signalShort)
                    signalShort = null;
            }
            else
            // Stop loss/Take profit filled
            if (orderState == OrderState.Filled)
            {
                if (order.FromEntrySignal == signalLong)
                    signalLong = null;
                else
                if (order.FromEntrySignal == signalShort)
                    signalShort = null;
            }
        }


        [Display(Name = "Take profit, ticks", GroupName = "00. Entry", Order = 1, Description = "")]
        [Range(1, int.MaxValue)]
        [NinjaScriptProperty]
        public int Strategy_TakeProfitTicks
        { get; set; } 

        [Display(Name = "Stop loss, ticks", GroupName = "00. Entry", Order = 2, Description = "")]
        [Range(1, int.MaxValue)]
        [NinjaScriptProperty]
        public int Strategy_StopLossTicks
        { get; set; } 

        /// <summary>
        /// Bid/Offer total volumes imbalance ratio.
        /// </summary>
        [Display(Name = "Imbalance Ratio", GroupName = "01. Strategy", Order = 1, Description = "Bid/Offer total volumes imbalance ratio.")]
        [Range(0, double.MaxValue)]
        [NinjaScriptProperty]
        public double Strategy_Ratio
        { get; set; }

        [Display(Name = "Block volume", GroupName = "01. Strategy", Order = 2, Description = "")]
        [Range(1, long.MaxValue)]
        [NinjaScriptProperty]
        public long Strategy_BlockVolumeFilter { get; set; } 

        [Display(Name = "Block distance, ticks", GroupName = "01. Strategy", Order = 3, Description = "")]
        [Range(1, int.MaxValue)]
        [NinjaScriptProperty]
        public int Strategy_BlockDistanceTicks { get; set; }

        [Display(Name = "Min trades number in cluster", GroupName = "01. Strategy", Order = 4, Description = "")]
        [Range(1, int.MaxValue)]
        [NinjaScriptProperty]
        public int Strategy_TradesNumber { get; set; }

        [Display(Name = "Cluster range", GroupName = "01. Strategy", Order = 5, Description = "")]
        [Range(1, int.MaxValue)]
        [NinjaScriptProperty]
        public int Strategy_ClusterRange { get; set; }
    }
}
#endif