﻿// MZpack 3 Pro API demo
// Copyright © 2020 MZpack Co.
// www.mzpack.pro
// WARNING: to compile this sample in NinjaTrader 8 remove '#if APISAMPLE' and '#endif' directives
#if APISAMPLE
using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using NinjaTrader.Data;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using MZpack;
using MZpack.NT8;
using MZpack.NT8.Algo;
using MZpack.NT8.Algo.Indicators;

namespace NinjaTrader.NinjaScript.Strategies.MZpackAPISamples
{
    /// <summary>
    /// Demonstrates how to use liquidity migration data of StrategyMarketDepthIndicator. Use it on 1 Tick chart.
    /// </summary>
    /// <summary>
    /// WARNING: The strategy works on Market replay or live data only.
    /// </summary>
    /// <summary>
    /// ATTENTION: Indicator settings in UI are used only for displaying markers on the chart. Strategy uses its own internal settings for filters and logic.
    /// </summary>
    /// <summary>
    /// See installation folder for the source code.
    /// </summary>
    [CategoryOrder("Entry", 10)]
    [CategoryOrder("Strategy", 20)]
    public class MZpackCustomStrategy6 : MZpackStrategyBase
    {
        StrategyMarketDepthIndicator marketDepthIndicator;
        string signalLong, signalShort;


        public MZpackCustomStrategy6() : base()
        {
            // Set OnCreateIndicators delegate
            OnCreateIndicators = new OnCreateIndicatorsDelegate(CreateIndicators);

            // Set OnBarCloseHandler for our on bar close strategy
            OnMarketDepthHandler = new OnMarketDepthDelegate(StrategyOnMarketDepthHandler);
        }

        /// <summary>
        /// Create indicators here and return them in the list as a result.
        /// </summary>
        /// <returns>Indicators list</returns>
        protected List<TickIndicator> CreateIndicators()
        {
            // Initialize new indicators list
            List<TickIndicator> indicators = new List<TickIndicator>();

            // Create indicator instance
            marketDepthIndicator = new StrategyMarketDepthIndicator(this, @"MarketDepth 6")
            {
                // Override defaults if required
                ShowHistoricalDOM = true,
                ShowImbalance = false,
                ShowRealtimeOrderBook = false,
                ShowCumulativeLevelII = false,

                // Migration
                ShowLiquidityMigration = true,

                // Display filters
                MarketDepthFilteringMode = MarketDepthFilteringMode.Percentage,

                // Presentation

                // Alerts
                UseAboveExpectedAlert = false,
            };
            indicators.Add(marketDepthIndicator);

            return indicators;
        }

        protected override void OnStateChange()
        {
            // Base OnStateChange() call is required
            base.OnStateChange();

            lock (Sync) // Sync OnStateChange handler
            {
                if (State == State.SetDefaults)
                {
                    EntriesPerDirection = 1;

                    // Set defaults
                    Entry_TakeProfitTicks = 4;
                    Entry_StopLossTicks = 4;
                    Strategy_MigrationAddedVolume = 40;
                    Strategy_MigrationBarsCount = 10;
                    Strategy_MigrationRatio = 1.5;
                }
                else
                if (State == State.Configure)
                {
                    // Set indicator attributes from UI properties
                    marketDepthIndicator.AddedVolumeFilter = Strategy_MigrationAddedVolume;
                    marketDepthIndicator.RemovedVolumeFilter = int.MaxValue; // Don't show removed liquidity
                }
            }
        }

        protected void StrategyOnMarketDepthHandler(MarketDepthEventArgs e, int currentBarIdx)
        {
            // Enter position
            if (Position.MarketPosition == Cbi.MarketPosition.Flat &&  // No position
                signalLong == null && signalShort == null)  // No orders has placed yet
            {
                MarketDepthBlocks blocks;
                if (marketDepthIndicator.Blocks.TryGetValue(CurrentBar, out blocks))  // Get DOM blocks for current bar
                {
                    // x.Value.Delta contains migration volume (positive or negative)
                    // Sum for Bid migrations
                    float bidMigration = blocks.Where(x => x.Value.Descriptor.Side == TradeSide.Bid 
                        && x.Value.Delta >= Strategy_MigrationAddedVolume 
                        && x.Value.Descriptor.StopBarIdx - x.Value.Descriptor.StartBarIdx >= Strategy_MigrationBarsCount).Sum(x => x.Value.Delta);
                    // Sum for Offer migrations
                    float offerMigration = blocks.Where(x => x.Value.Descriptor.Side == TradeSide.Ask 
                        && x.Value.Delta > Strategy_MigrationAddedVolume
                        && x.Value.Descriptor.StopBarIdx - x.Value.Descriptor.StartBarIdx >= Strategy_MigrationBarsCount).Sum(x => x.Value.Delta);

                    if (bidMigration > 0 || offerMigration > 0)
                    {
                        float ratio;
                        // Calculate ratio of migrations
                        if (bidMigration == 0 || offerMigration == 0)
                            ratio = int.MaxValue;
                        else
                            ratio = Math.Max(bidMigration, offerMigration) / Math.Min(bidMigration, offerMigration); 

                        if (ratio >= Strategy_MigrationRatio)
                        {
                            // Long signal
                            if (bidMigration > offerMigration)
                            {
                                signalLong = "LONG. Ratio:" + (ratio < int.MaxValue ? string.Format("{0:N2}", ratio) : "INF");
                                SetProfitTarget(signalLong, CalculationMode.Ticks, Entry_TakeProfitTicks);
                                SetStopLoss(signalLong, CalculationMode.Ticks, Entry_StopLossTicks, false);
                                EnterLong(signalLong);
                            }
                            // Short signal
                            else
                            {
                                signalShort = "SHORT. Ratio:" + (ratio < int.MaxValue ? string.Format("{0:N2}", ratio) : "INF");
                                SetProfitTarget(signalShort, CalculationMode.Ticks, Entry_TakeProfitTicks);
                                SetStopLoss(signalShort, CalculationMode.Ticks, Entry_StopLossTicks, false);
                                EnterShort(signalShort);
                            }
                        }
                    }
                }
            }
        }

        protected override void OnOrderUpdate(Order order, double limitPrice, double stopPrice, int quantity, int filled, double averageFillPrice, OrderState orderState, DateTime time, ErrorCode error, string comment)
        {
            // Order cancelled/rejected
            if (orderState == OrderState.Cancelled || orderState == OrderState.CancelSubmitted || orderState == OrderState.Rejected)
            {
                if (order.Name == signalLong)
                    signalLong = null;
                else
                if (order.Name == signalShort)
                    signalShort = null;
            }
            else
            // Stop loss/Take profit filled
            if (orderState == OrderState.Filled)
            {
                if (order.FromEntrySignal == signalLong)
                    signalLong = null;
                else
                if (order.FromEntrySignal == signalShort)
                    signalShort = null;
            }
        }


        [Display(Name = "Take profit, ticks", GroupName = "Entry", Order = 1, Description = "")]
        [Range(1, int.MaxValue)]
        [NinjaScriptProperty]
        public int Entry_TakeProfitTicks
        { get; set; }

        [Display(Name = "Stop loss, ticks", GroupName = "Entry", Order = 2, Description = "")]
        [Range(1, int.MaxValue)]
        [NinjaScriptProperty]
        public int Entry_StopLossTicks
        { get; set; }

        [Display(Name = "Migration filter", GroupName = "Strategy", Order = 1, Description = "")]
        [Range(1, int.MaxValue)]
        [NinjaScriptProperty]
        public int Strategy_MigrationAddedVolume
        { get; set; }
        
        /// <summary>
        /// How many bars block with migration has.
        /// </summary>
        [Display(Name = "Migration bars", GroupName = "Strategy", Order = 2, Description = "")]
        [Range(1, int.MaxValue)]
        [NinjaScriptProperty]
        public int Strategy_MigrationBarsCount
        { get; set; }

        /// <summary>
        /// Ratio of liquidity migration on Bid/Offer
        /// </summary>
        [Display(Name = "Migration ratio", GroupName = "Strategy", Order = 3, Description = "")]
        [Range(0, double.MaxValue)]
        [NinjaScriptProperty]
        public double Strategy_MigrationRatio
        { get; set; }

    }
}
#endif
